@@ -5,6 +5,7 @@ import pickle
 import pandas as pd
 import utilities as ut
 import sklearn.manifold as man
+import sklearn.utils.sparsefuncs as sf
 import warnings
 
 warnings.filterwarnings("ignore", message="numpy.dtype size changed")
@@ -19,7 +20,7 @@ except ImportError:
     PLOTTING = False
 
 
-__version__ = '0.3.3'
+__version__ = '0.3.5'
 
 """
 Copyright 2018, Alexander J. Tarashansky, All rights reserved.
@@ -36,9 +37,16 @@ class SAM(object):
 
     Parameters
     ----------
-    counts : tuple (scipy.sparse matrix, numpy.array,numpy.array), optional, default None
-        A tuple containing sparse data structure of the gene expression counts
-        (cells x genes), numpy array of gene IDs, and numpy array of cell IDs.
+    counts : tuple or list (scipy.sparse matrix, numpy.ndarray,numpy.ndarray), OR
+        tuple or list (numpy.ndarray, numpy.ndarray,numpy.ndarray), OR
+        pandas.DataFrame
+
+        If a tuple or list, it should contain the gene expression data (scipy.sparse or 
+        numpy.ndarray) matrix (cells x genes), numpy array of gene IDs, and numpy array
+        of cell IDs in that order.
+
+        If a pandas.DataFrame, it should be (cells x genes)
+
         Only use this argument if you want to pass in preloaded data. Otherwise
         use one of the load functions.
 
@@ -136,16 +144,28 @@ class SAM(object):
     def __init__(self, counts=None, annotations=None, k=20,
                  distance='correlation'):
         
-        if counts is not None:
+        if isinstance(counts, tuple) or isinstance(counts,list):
             self.sparse_data,self.all_gene_names,self.all_cell_names = counts
+            if isinstance(self.sparse_data,np.ndarray):
+                self.sparse_data = sp.csr_matrix(self.sparse_data)
             self.D = self.sparse_data.copy()
-            self.gene_names=self.all_gene_names
-            self.cell_names=self.all_cell_names
-                  
-         
-        
+            self.all_gene_names=np.array(list(self.all_gene_names))
+            self.all_cell_names=np.array(list(self.all_cell_names))
+            self.gene_names=self.all_gene_names.copy()
+            self.cell_names=self.all_cell_names.copy()
+        elif isinstance(counts,pd.DataFrame):
+            self.sparse_data = sp.csr_matrix(counts.values)
+            self.all_gene_names = np.array(list(counts.columns.values))
+            self.all_cell_names = np.array(list(counts.index.values))
+            self.D = self.sparse_data.copy()
+            self.gene_names=self.all_gene_names.copy()
+            self.cell_names=self.all_cell_names.copy()
+        elif counts is not None:
+            raise Exception("\'counts\' must be either a tuple/list of (data,gene IDs,cell IDs)"
+                             "or a Pandas DataFrame of cells x genes")
+
         if(annotations is not None):
-            self.annotations = annotations
+            self.annotations = np.array(list(annotations))
             self.integer_annotations = ut.convert_annotations(self.annotations)
 
         self.k = k
@@ -518,15 +538,9 @@ class SAM(object):
 
         D_avg = (nnm/self.k).dot(self.D2)
         
-        self.D_avg = D_avg.copy()
+        self.D_avg = D_avg
+        mu,var = sf.mean_variance_axis(D_avg,axis=0)
 
-        mu = D_avg.mean(0)
-        Ex2=np.square(mu) 
-        D_avg.data=D_avg.data**2
-        Ex1=D_avg.mean(0)
-        var = np.array(Ex1-Ex2).flatten()
-        mu = np.array(mu).flatten()
-        
         dispersions = np.zeros(var.size)
         dispersions[mu>0] = var[mu>0]/mu[mu>0]
         
@@ -889,12 +903,14 @@ class SAM(object):
             
         """
         self._create_dict(exc)
-
+        if savename[-2:] != '.p':
+            savename = savename + '.p'
+            
         if(dirname is not None):
             ut.create_folder(dirname+"/")
-            f = open(dirname+"/" + savename + ".p", 'wb')
+            f = open(dirname+"/" + savename, 'wb')
         else:
-            f = open(savename + ".p", 'wb')
+            f = open(savename, 'wb')
 
         pickle.dump(self.pickle_dict, f)
         f.close()
